#ifndef BOOKMOOCHERCORE_H
#define BOOKMOOCHERCORE_H

#include <QObject>
#include <QFile>
#include <QNetworkAccessManager>
#include <QDateTime>
#include <QTimer>

#include "defines.h"
#include "BookListItem.h"
#include "booklistmodel.h"

#include "BookInfoItem.h"

/*
	Contiene le funzionalità base
*/
class BookMoocherCore : public QObject
{
    Q_OBJECT
public:

    enum BookMoocherUpdateStatus { UpdateOK, UpdateFailed, NetworkError } ;

    /**
     * @throws BMException
     */
    explicit BookMoocherCore(QObject *parent = 0);

	/**
	 * Applica i filtri prendendo le informazioni dai settaggi dell'applicazione
	 */
    void applyFilters(BookList &book_list);

	/**
	 * Filtra una lista di libri in base a nazione, data.
	 *
	 * Permette inoltre di limitare il numero di elementi.
	 * Gli elementi vengono prima ordinati per data (in modo discendente) e poi ridotti, in modo da
	 * avere solo gli N elementi più recenti.
	 */
	void applyFilters(BookList &book_list, QStringList &countries, QDateTime &remove_before, int limit_entries = -1, Qt::SortOrder order = Qt::DescendingOrder);


	bool isUpdating() { return is_updating; }

	static QString getLastFeedFilename();
	static QString getBookListFilename();

	// Salva una lista di libri nel file standard
	bool saveBookList(const BookList &book_list, const QDateTime &fetch_date, QString& error_msg);

	// Salva una lista di libri nel file specifico
	bool saveBookList(const QString &out_filepath, const BookList &book_list, const QDateTime &fetch_date, QString& error_msg);

	// Carica la lista di libri dal file standard nel model
	void loadBookList(BookListModel *model);

	// Carica la lista di libri dal file specifico
	bool loadBookList(const QString &in_filepath, BookList &book_list, QDateTime &list_update_time, QString &version);

	// Converte il formato su disco della lista dei libri alla versione più recente
	// e ritorna la lista aggiornata
	bool convertBookListToLatestVersion(const QString &in_filepath, BookList &book_list, QString &error_msg);
	bool convertBookListToLatestVersion(BookListModel *model, QString &error_msg);

	// Nel caso la richiesta di aggiornamento abbia ritornato un errore
	// contiene l'errore di rete riportato. è una delle costanti di errore di QNetworkReply::NetworkError
	int getLastNetworkReplyError() { return last_network_reply_error; }

	// Ritorna (se esiste) l'ora dell'ultimo aggiornamento del feed
	const QDateTime &getLastUpdateTime() const;

signals:
	void finished(int new_element_count, BookMoocherCore::BookMoocherUpdateStatus status = UpdateOK, const QString &error_msg = QString());
	void finished(int new_element_count, BookList &book_list, BookMoocherCore::BookMoocherUpdateStatus status = UpdateOK, const QString &error_msg = QString());

	void finishedBookInfoFetch(BookInfoItem &bookinfo, BookMoocherCore::BookMoocherUpdateStatus status = UpdateOK, const QString& error_msg = QString());

public slots:
	// Scarica il feed XML con le informazioni
	void fetchBookmoochXMLFeed();

	// Scarica le informazioni su un libro
	void fetchBookInfo(const QString& ASIN);
	void fetchBookInfo(const BookListItem& item);

private:
	// Gestore della connessione
   QNetworkAccessManager *qnam;
   QNetworkAccessManager *book_info_qnam;

   // Model dei dati
   BookListModel *model;

   // Ora dell'ultimo aggiornamento
   QDateTime last_update_time;

   // Timer per l'aggiornamento
   QTimer *timer;

   // flag che indica se sta aggiornando
   bool is_updating;

   // Stato dell'ultima operazione di fetch, indica un valore dell'enum QNetworkReply::NetworkError
   int last_network_reply_error;

   /**
    * Esegue il parsing di una sorgente XML contenente il feed dei libri.
    */
	bool parseXML(QIODevice *xml_source, int& new_elements_count, BookList &new_book_list, QString &error_msg );

	/**
	 * Salva il file XML con il feed dei libri
	 */
	bool saveBookFeed(QIODevice *xml_source, QString &error_msg);

	/*
	 * Esegue il parsing dell'XML di descrizione di un libro
	 *
	 * @return TRUE se tutto ok, FALSE in caso di errore
	 * */
	bool parseBookInfoXML(QIODevice *source, BookInfoItem *book_info, QString &err_msg);

private slots:
   void finishedBookmoochXMLFetch(QNetworkReply *);

   void finishedBookInfoXMLFetch(QNetworkReply*);

};

#endif // BOOKMOOCHERCORE_H
